@@ -1,13 +1,26 @@
 import sys
 import inspect
 from functools import partial
-from .cross import PY2
+from .compat import PY2
 
 
 __all__ = ['decorator', 'wraps', 'unwrap', 'ContextDecorator', 'contextmanager']
 
 
 def decorator(deco):
+    """
+    Transforms a flat wrapper into decorator::
+
+        @decorator
+        def func(call, methods, content_type=DEFAULT):  # These are decorator params
+            # Access call arg by name
+            if call.request.method not in methods:
+                # ...
+            # Decorated functions and all the arguments are accesible as:
+            print(call._func, call_args, call._kwargs)
+            # Finally make a call:
+            return call()
+    """
     if has_single_arg(deco):
         return wraps(deco)(make_decorator(deco))
     else:
@@ -72,10 +85,12 @@ def arggetter(func, _cache={}):
     if func in _cache:
         return _cache[func]
 
-    argnames = get_argnames(func)
+    original = getattr(func, '__original__', None) or unwrap(func)
+    argnames = get_argnames(original)
     indexes = dict((name, i) for i, name in enumerate(argnames))
-    if func.__defaults__:
-        defaults = dict((argnames[-1-i], v) for i, v in enumerate(func.__defaults__))
+    defaults_tuple = original.__defaults__
+    if defaults_tuple:
+        defaults = dict(zip(argnames[-len(defaults_tuple):], defaults_tuple))
     else:
         defaults = {}
 
@@ -176,7 +191,7 @@ else:
                     # was passed to throw().  This prevents a StopIteration
                     # raised inside the "with" statement from being suppressed
                     return exc is not value
-                except:
+                except:  # noqa
                     # only re-raise if it's *not* the exception that was
                     # passed to throw(), because __exit__() must not raise
                     # an exception unless __exit__() itself failed.  But throw()
@@ -197,9 +212,9 @@ else:
 
             @contextmanager
             def tag(name):
-                print "<%s>" % name,
+                print("<%s>" % name)
                 yield
-                print "</%s>" % name
+                print("</%s>" % name)
 
             with tag("h1"):
                 print "foo",
@@ -209,7 +224,7 @@ else:
 
             @tag('strong')
             def shout(text):
-                print text.upper()
+                print(text.upper())
 
             shout('hooray')
             # -> <strong> HOORAY </strong>
@@ -266,6 +281,7 @@ def wraps(wrapped,
             def wrapper(*args, **kwargs):
                 do_something(*args, **kwargs)
                 return func(*args, **kwargs)
+            return wrapper
 
     But see also :func:`@decorator<decorator>` for that.
     This is extended version of :func:`functools.wraps`.
